using System;
using UnityEngine;
using UnityEngine.Events;
using UnityStandardAssets.CrossPlatformInput;
using UnityStandardAssets._2D;
using System.Collections;

public class PlatformDepth : MonoBehaviour
{
    public float animationTime = 2;
    public int curretDepth = 0;

    private Vector3 startPosition;
    private Vector3 targetPosition;
    private bool moving = false;
    private bool hasTarget = false;
    private float currentTime = 0;
    private Rigidbody body;
    private Animator animator;
    private SpriteRenderer[] spriteRenderer;
    private string[] layers = new string[] { "Z-1", "Z0", "Z1" };


    private void setLayer()
    {
        for (int i = 0; i < spriteRenderer.Length; i++)
        {
            spriteRenderer[i].sortingLayerName = layers[curretDepth];
        }
    }

    private void Start()
    {
        body = GetComponent <Rigidbody>();
        animator = GetComponent<Animator>();
        spriteRenderer = GetComponentsInChildren<SpriteRenderer>();
    }

    private void FixedUpdate()
    {	
        if (moving)
        {
            currentTime += Time.deltaTime;
            transform.position = Vector3.Lerp(startPosition, targetPosition, currentTime / animationTime);
            if (currentTime > animationTime)
            {
                setLayer();
                body.isKinematic = false;
                moving = false;
                animator.SetBool("BackRun", false);
                animator.SetBool("FrontRun", false);
                return;
            }
        }
    }

    public bool Interact(int direction)
    {
        if (!moving && hasTarget && (direction * (targetPosition.z - transform.position.z)) > 0)
        {
            if (direction > 0)
            {
				curretDepth += direction;
                animator.SetBool("BackRun", true);
                
            }
            else
            {
				curretDepth += direction;
				setLayer();
                animator.SetBool("FrontRun", true);
            }
            body.isKinematic = true;
            moving = true;
            startPosition = transform.position;
            currentTime = 0;
            return true;
        }
        return false;
    }

    public void SetTarget(Vector3 target)
    {
        hasTarget = true;
        targetPosition = target;    
    }

    public void ClearTarget()
    {
        hasTarget = false;
    }

    public bool IsMoving()
    {
        return moving;
    }

}

