﻿using UnityEngine;
using System.Collections;
using UnityStandardAssets._2D;

public class Vine : MonoBehaviour {

	public float timePull;
	public float timeRelease;
	public Transform pullTarget;

	private Rigidbody body;
	private Collider col;
	private Vector3 startPosition;
	private Vector3 targetPosition;
	private bool pooling;
	private FixedJoint distanceJoint;
	private Platformer2DUserControl player;

	// Use this for initialization
	void Start () {
		body = GetComponent<Rigidbody> ();
		col = GetComponent<Collider> ();
		startPosition = transform.position;
		targetPosition = pullTarget.position;
	}

	void OnTriggerEnter(Collider other)
	{
		if (other.tag == "Player" && !pooling) {
			player = other.GetComponent<Platformer2DUserControl> ();
			player.enabled = false;
			col.enabled = false;
			pooling = true;
			distanceJoint = gameObject.AddComponent<FixedJoint>();
			distanceJoint.connectedBody = other.GetComponent<Rigidbody>();
			LeanTween.moveY (gameObject, targetPosition.y, timePull).setOnComplete (MovedToBottom);
		} 
	}

	void MovedToBottom() {
		player.enabled = true;
		Destroy (distanceJoint);
		LeanTween.moveY (gameObject, startPosition.y, timeRelease).setOnComplete (MovedToTop);
	}

	void MovedToTop() {
		col.enabled = true;
		pooling = false;
	}

	public float PercentPulled() {
		return Mathf.Abs (transform.position.y - startPosition.y ) / Mathf.Abs (targetPosition.y - startPosition.y);
	}

}
